var h1e = h1edpi_module
var fl = Math.floor
var rand = Math.random

h1e.default_text_bgcolor = undefined
h1e.default_text_fgcolor = "#888888"
h1e.default_text_fontsize = 8
h1e.default_text_fonttype = "native"
h1e.default_text_native_fontname = "monospace"
h1e.default_text_sprite_fontname = "font"

var FPS = 30
var TICK_LENGTH = 1000/FPS
var SCREEN_W = 240
var SCREEN_H = 160
var GEN_W = 9
var GEN_H = 9
var LEVEL_SELECTED_I = 0
var DESTRUCT_TIME = 1.0
var BEAM_TIME = 0.3

h1e.bgstyle = "#000000" // Set to undefined if background image is used
h1e.init($("#main_canvas")[0], SCREEN_W, SCREEN_H, FPS)

//h1e.add_image("background", "background.png")
//h1e.def_sprite("background", "background", [[0,0,240,160]])

h1e.add_image("sprites", "sprites.png")
var toprow = ["tile_bg", "tile_empty", ["tile_machine", 3],
		"tile_block", "tile_limit"]
var x = 0
toprow.forEach(function(name, i){
	var sprite_name = undefined
	var frames = []
	if(h1e.isstring(name)){
		sprite_name = name
		frames.push([x,0,20,30])
		x += 20
	} else if(h1e.isarray(name)){
		sprite_name = name[0]
		for(var j=0; j<name[1]; j++){
			frames.push([x,0,20,30])
			x += 20
		}
	} else {
		throw "Invalid sprite definition"
	}
	h1e.def_sprite(sprite_name, "sprites", frames, [10,24])
})
var row2 = ["player_left", "player_right", "player_up", "player_down",
		"tile_destruct"]
row2.forEach(function(name, i){
	h1e.def_sprite(name, "sprites", [[i*20,30,20,30]], [10,24])
})
var row3 = ["small_bg", "small_empty", "small_player",
		"small_machine", "small_block", "small_limit"]
row3.forEach(function(name, i){
	h1e.def_sprite(name, "sprites", [[i*3,60,3,3]], [0,0])
})
var row4 = ["beam_left", "beam_right", "beam_up", "beam_down"]
row4.forEach(function(name, i){
	h1e.def_sprite(name, "sprites", [[i*20,63,20,27]], [10,21])
})
var row5 = ["beam2_left", "beam2_right", "beam2_up", "beam2_down"]
row5.forEach(function(name, i){
	h1e.def_sprite(name, "sprites", [[i*20,93,20,27]], [10,21])
})

function diri_to_dirname(diri){ return (["left","right","up","down"])[diri] }
function dirname_to_diri(dirname){ return {left:0,right:1,up:2,down:3}[dirname] }
function diri_to_dirv(diri){
	if(diri < 0 || diri > 3) return {x:0, y:0}
	var a = [-1,0, 1,0, 0,-1, 0,1]
	return {x:a[diri*2+0], y:a[diri*2+1]}
}
function dirv_to_diri(dirv){
	if(dirv.x ==-1 && dirv.y == 0) return 0
	if(dirv.x == 1 && dirv.y == 0) return 1
	if(dirv.x == 0 && dirv.y ==-1) return 2
	if(dirv.x == 0 && dirv.y == 1) return 3
	return -1
}
function diri_reverse(diri){ return ([1,0,3,2])[diri] }

function char_to_tiletype(c){
	return {" ":"bg", "p":"player", "m":"machine",
			"b":"block", "l":"limit"}[c]
}

function random_frame_at_xy(num_frames, x, y){
	return (x * 11 + y * 7) % num_frames
}

function Tile(type){
	this.type = type || "empty"
}

// Only holds game state
function Game(){
	var game = this
	game.w = 0
	game.h = 0
	game.tiles = []
	game.player_x = 4
	game.player_y = 8
	game.old_player_x = game.player_x
	game.old_player_y = game.player_y
	game.move_count = 0
	game.left_count = 0
	game.right_count = 0
	game.back_count = 0
	game.last_diri = dirname_to_diri("up")
	game.last_last_diri = game.last_diri
	game.trigger_base = 2
	game.trigger_add = 0

	game.feedback = {}
	game.feedback.original_level = undefined
	game.feedback.last_tiles_destructed = []
	game.feedback.completed = false
	game.feedback.beams = []
}

// level: type="generate"
// level: type="saved", data={w,h,tiles="p mbl"}
function create_game_from_level(level){
	var game = new Game()
	if(level.type === "generate"){
		level = generate_saved_level(GEN_W, GEN_H)
		// Load level as saved
	}
	if(level.type === "saved"){
		var data = level.data
		game.w = data.w
		game.h = data.h
		for(var y1=0; y1<data.h; y1++)
		for(var x1=0; x1<data.w; x1++){
			var i1 = y1*data.w + x1
			var t = data.tiles[i1]
			var type = char_to_tiletype(t)
			if(t === "p"){
				game.player_x = x1
				game.player_y = y1
				game.old_player_x = game.player_x
				game.old_player_y = game.player_y
				type = "empty"
			}
			else if(type === "bg"){
				type = "empty"
			}
			game.tiles.push(new Tile(type))
		}
		game.feedback.original_level = level
	}
	else {
		throw "Invalid level type \""+level.type+"\""
	}
	return game
}

function update_trigger(game){
	var trigger_add = 0
	var num_target_tiles = 0
	for(var y=0; y<game.h; y++)
	for(var x=0; x<game.w; x++){
		var i = y*game.w + x
		var t = game.tiles[i]
		if(t.type === "limit"){
			trigger_add += 1
		}
		if(t.type !== "empty" && t.type !== "block"){
			num_target_tiles += 1
		}
	}
	game.trigger_add = trigger_add
	if(num_target_tiles === 0 && !game.feedback.completed){
		game.feedback.completed = true

		// Send to server
		if(window.location.protocol !== "file:"){
			$.ajax({
				type: "POST",
				url: "/level_completed",
				data: JSON.stringify({
					level: game.feedback.original_level,
					moves: game.move_count,
				}),
				contentType: "application/json",
				success: function(data){
				},
				error: function(xhr, type){
				}
			});
		}
	}
}

function shoot_beam(game, p0, v, tiles_destructed, opts){
	opts = opts || {}
	var p = {x:p0.x, y:p0.y}
	var diri = dirv_to_diri(v)
	for(;;){
		p.x += v.x
		p.y += v.y
		if(p.x < 0 || p.x >= game.w || p.y < 0 || p.y >= game.h)
			break
		var i = p.y*game.w + p.x
		var t = game.tiles[i]
		if(t.type === "block" && !opts.destroy_block){
			break
		}
		game.feedback.beams.push({
			x: p.x,
			y: p.y,
			diri: diri,
			timer: 0,
			opts: opts,
		})
		if(t.type !== "empty"){
			if(tiles_destructed)
				tiles_destructed.push({x:p.x, y:p.y, type:t.type})
			t.type = "empty"
		}
	}
}

function do_beams(game, allowed_beams){
	var trigger = game.trigger_base + game.trigger_add
	var tiles_destructed = []
	var beam_shot = false

	if(game.back_count >= trigger && allowed_beams.back){
		game.back_count = 0
		var dirv0 = diri_to_dirv(game.last_diri)
		var beamv = {x:dirv0.y, y:dirv0.x}
		shoot_beam(game, {x:game.old_player_x, y:game.old_player_y}, beamv,
				tiles_destructed, {destroy_block:true})
		var beamv = {x:-dirv0.y, y:-dirv0.x}
		shoot_beam(game, {x:game.old_player_x, y:game.old_player_y}, beamv,
				tiles_destructed, {destroy_block:true})
		beam_shot = true
	}

	update_trigger(game)
	var trigger = game.trigger_base + game.trigger_add

	if((game.left_count >= trigger && allowed_beams.left) ||
			(game.right_count >= trigger && allowed_beams.right)){
		var diri = game.last_last_diri
		var dirv = diri_to_dirv(diri)
		var diru = {x:dirv.y, y:dirv.x}
		var p = {x:game.old_player_x, y:game.old_player_y}
		for(;;){
			if(p.x < 0 || p.x >= game.w || p.y < 0 || p.y >= game.h)
				break
			var i = p.y*game.w + p.x
			var t = game.tiles[i]
			if(t.type === "empty"){
				shoot_beam(game, p, dirv, tiles_destructed)
			}
			p.x += diru.x
			p.y += diru.y
		}
		var p = {x:game.old_player_x, y:game.old_player_y}
		for(;;){
			p.x -= diru.x
			p.y -= diru.y
			if(p.x < 0 || p.x >= game.w || p.y < 0 || p.y >= game.h)
				break
			var i = p.y*game.w + p.x
			var t = game.tiles[i]
			if(t.type === "empty"){
				shoot_beam(game, p, dirv, tiles_destructed)
			}
		}
		beam_shot = true
	}

	if(tiles_destructed.length > 0){
		game.feedback.last_tiles_destructed = tiles_destructed
		game.feedback.last_tiles_destructed_timer = 0
	}

	update_trigger(game)
	var trigger = game.trigger_base + game.trigger_add

	if(game.left_count >= trigger && allowed_beams.left)
		game.left_count = 0
	if(game.right_count >= trigger && allowed_beams.right)
		game.right_count = 0

	if(beam_shot){
		var r = Math.random()
		if(r < 0.33)
			$("#audio_prrww1")[0].play()
		else if(r < 0.66)
			$("#audio_prrww2")[0].play()
		else
			$("#audio_prrww3")[0].play()
	}
}

function try_move_player(game, dx, dy){
	var new_x = game.player_x + dx
	var new_y = game.player_y + dy
	if(new_x < 0 || new_x >= game.w || new_y < 0 || new_y >= game.h)
		return
	var i = new_y*game.w + new_x
	var t = game.tiles[i]
	if(t.type !== "empty")
		return
	game.old_player_x = game.player_x
	game.old_player_y = game.player_y
	game.player_x = new_x
	game.player_y = new_y
	game.move_count += 1

	var dirv0 = diri_to_dirv(game.last_diri)
	var dirv1 = {x:dx, y:dy}

	// Normalize dirv1->dirv1n to dirv0n=(0,-1)
	var dirv1n = (function(){
		if(dirv0.y < 0)
			return {x:dirv1.x, y:dirv1.y}
		if(dirv0.y > 0)
			return {x:-dirv1.x, y:-dirv1.y}
		if(dirv0.x < 0)
			return {x:-dirv1.y, y:dirv1.x}
		if(dirv0.x > 0)
			return {x:dirv1.y, y:-dirv1.x}
		return {x:0, y:0}
	})()

	var allowed_beams = {}
	if(dirv1n.y > 0){
		game.back_count += 1
		allowed_beams.back = true
	}
	if(dirv1n.x < 0){
		game.left_count += 1
		allowed_beams.left = true
	}
	if(dirv1n.x > 0){
		game.right_count += 1
		allowed_beams.right = true
	}

	var diri = dirv_to_diri(dirv1)
	game.last_last_diri = game.last_diri
	game.last_diri = diri

	do_beams(game, allowed_beams)
}

function GameSection(game){
	var section = this
	if(!game){
		var level = generate_saved_level(GEN_W, GEN_H)
		game = create_game_from_level(level)
	}
	this.game = game
	update_trigger(game)

	this.reset_game = function(new_game){
		game = new_game
		this.game = game
		update_trigger(game)
	}
	this.draw = function(h1e){
		//h1e.draw_sprite(0, 0, "background")
		var y0 = 0
		h1e.draw_text(160, y0+8, "Moves:")
		h1e.draw_text(210, y0+8, ""+game.move_count, {right_align:true})
		if(game.feedback.original_level &&
				game.feedback.original_level.best_moves !== undefined){
			var best_moves = game.feedback.original_level.best_moves
			h1e.draw_text(160, y0+16, "   Best:",
					{fgcolor:"#333333", fontsize:6})
			h1e.draw_text(210-0.5, y0+16, ""+best_moves,
					{right_align:true, fgcolor:"#333333", fontsize:6})
			/*h1e.draw_text(210, y0+16, "(best)",
					{fgcolor:"#333333", fontsize:6})*/
		}
		if(game.feedback.completed){
			var opts = {fgcolor:"#ffffff"}
			h1e.draw_text(160, y0+28, "Level completed!", opts)
		}
		var y0 = 50
		var trigger = game.trigger_base+game.trigger_add
		h1e.draw_text(160, y0+8*0, "Trigger: "+game.trigger_base+"+"+
				game.trigger_add+"="+trigger)
		var y0 = 70
		var hl_opts = game.feedback.completed ? undefined : {fgcolor:"#ffffff"}
		h1e.draw_text(160, y0+8*0, " Left: "+game.left_count,
				game.left_count >= trigger-1 ? hl_opts : undefined)
		h1e.draw_text(160, y0+8*1, "Right: "+game.right_count,
				game.right_count >= trigger-1 ? hl_opts : undefined)
		h1e.draw_text(160, y0+8*2, " Back: "+game.back_count,
				game.back_count >= trigger-1 ? hl_opts : undefined)
		var opts = {fontsize:6, fgcolor:"#333333"}
		var hl_opts = {fontsize:6, fgcolor:"#ffffff"}
		h1e.draw_text(160, 160-6*8, "  --- Objective ---", opts)
		h1e.draw_text(160, 160-6*7, " Destroy all fungus!", opts)
		h1e.draw_text(160, 160-6*5, "  --- Controls ---", opts)
		h1e.draw_text(165, 160-6*4, "Move:", opts)
		h1e.draw_text(190, 160-6*4, "Arrow keys", opts)
		h1e.draw_text(165, 160-6*3, "Level select: L",
				game.feedback.completed ? hl_opts : opts)
		h1e.draw_text(165, 160-6*2, "     restart: R",
				game.feedback.completed ? hl_opts : opts)
		var gx0 = 10
		var gy0 = 16
		var d = Math.sqrt(game.feedback.last_tiles_destructed.length)
		var t = d * 0.1
		if(t > DESTRUCT_TIME)
			t = DESTRUCT_TIME
		if(game.feedback.last_tiles_destructed_timer < t){
			gx0 += (Math.random()*2-0.5) * 2//d
			gy0 += (Math.random()*2-0.5) * 2//d
		}
		for(var y=0; y<game.h; y++)
		for(var x=0; x<game.w; x++){
			var i = y*game.w + x
			h1e.draw_sprite(x*16+gx0, y*16+gy0, "tile_bg")
		}
		for(var y=0; y<game.h; y++)
		for(var x=0; x<game.w; x++){
			var i = y*game.w + x
			var t = game.tiles[i]
			var sprite_name = "tile_"+t.type
			var num_frames = h1e.get_frame_count(sprite_name)
			h1e.draw_sprite(x*16+gx0, y*16+gy0, "tile_"+t.type,
					{frame:random_frame_at_xy(num_frames, x, y)})
		}
		var d = game.feedback.last_tiles_destructed_timer / DESTRUCT_TIME
		if(d < 1.0){
			var a = 1.0 - Math.sqrt(d)
			for(var i=0; i<game.feedback.last_tiles_destructed.length; i++){
				var t = game.feedback.last_tiles_destructed[i]
				var x = t.x
				var y = t.y
				var sprite_name = "tile_"+t.type
				var num_frames = h1e.get_frame_count(sprite_name)
				h1e.draw_sprite(x*16+gx0, y*16+gy0, "tile_"+t.type, {
					frame: random_frame_at_xy(num_frames, x, y),
					alpha: a,
				})
				h1e.draw_sprite(x*16+gx0, y*16+gy0, "tile_destruct")
			}
		}
		if(game.feedback.completed && game.feedback.original_level &&
				game.feedback.original_level.type === "saved"){
			var data = game.feedback.original_level.data
			var x0 = 210
			var y0 = 68
			for(var y1=0; y1<data.h; y1++)
			for(var x1=0; x1<data.w; x1++){
				var i1 = y1*data.w + x1
				var t = data.tiles[i1]
				var tn = char_to_tiletype(t)
				h1e.draw_sprite(x0+x1*3, y0+y1*3, "small_"+tn)
			}
		}
		for(var i=0; i<game.feedback.beams.length; i++){
			var beam = game.feedback.beams[i]
			//var a = 1.0 - Math.sqrt(beam.timer / BEAM_TIME)
			//var opts = {alpha: a}
			var opts = {}
			var dirname = diri_to_dirname(beam.diri)
			var sprite_name = "beam_"+dirname
			if(Math.floor(beam.timer*10)%2 == 1)
				sprite_name = "beam2_"+dirname
			h1e.draw_sprite(beam.x*16+gx0, beam.y*16+gy0, sprite_name, opts)
		}
		var x = game.player_x
		var y = game.player_y
		var dirname = diri_to_dirname(game.last_diri)
		if(!dirname)
			dirname = "up"
		h1e.draw_sprite(x*16+gx0, y*16+gy0, "player_"+dirname)
	}
	this.event = function(h1e, event){
		if(event.type == "keydown_repeatable"){
			if(!game.feedback.completed){
				if(h1e.iskey(event.key, ["left", "a"])){
					try_move_player(game, -1, 0)
					return true
				}
				if(h1e.iskey(event.key, ["right", "d"])){
					try_move_player(game, 1, 0)
					return true
				}
				if(h1e.iskey(event.key, ["up", "w"])){
					try_move_player(game, 0, -1)
					return true
				}
				if(h1e.iskey(event.key, ["down", "s"])){
					try_move_player(game, 0, 1)
					return true
				}
			}
			if(h1e.iskey(event.key, ["r"])){
				var level = game.feedback.original_level
				if(game.feedback.completed){
					if(level.best_moves === undefined ||
							game.move_count < level.best_moves){
						level.best_moves = game.move_count
					}
				}
				var new_game = create_game_from_level(level)
				section.reset_game(new_game)
				return true
			}
			if(h1e.iskey(event.key, ["l", "escape", "pageup"])){
				h1e.push_section(new LevelSelectSection(section))
				return true
			}
			if(game.feedback.completed){
				if(h1e.iskey(event.key, ["enter", "space"])){
					h1e.push_section(new LevelSelectSection(section))
					return true
				}
			}
		}
		/*if(event.type == "mousedown"){
			var mx = h1e.mousex()
			var my = h1e.mousey()
			return true
		}*/
		/*if(event.type == "mouseup"){
			return true
		}*/
	}
	this.update = function(h1e){
		if(game.feedback.last_tiles_destructed_timer < DESTRUCT_TIME){
			h1e.trigger_redraw()
			game.feedback.last_tiles_destructed_timer += 1/FPS
		}
		game.feedback.beams = game.feedback.beams.filter(function(beam){
			h1e.trigger_redraw()
			beam.timer += 1/FPS
			if(beam.timer > BEAM_TIME)
				return false
			return true
		})
		return false // Always redraw manually
	}
}

function LevelSelectSection(game_section){
	var section = this
	section.h1e_pass_draw = false
	section.h1e_pass_update = false
	section.h1e_pass_event = false

	section.local_levels = [
		{
			type: "generate",
		},
	]
	section.server_levels = undefined
	section.levels = []
	section.selected_i = LEVEL_SELECTED_I

	section.levels_per_row = 8

	section.update_levels = function(){
		section.levels = []
		section.local_levels.forEach(function(level){
			section.levels.push(level)
		})
		if(section.server_levels){
			section.server_levels.forEach(function(server_level){
				section.levels.push(server_level)
			})
		}
	}

	section.update_levels()

	if(window.location.protocol !== "file:"){
		$.ajax({
			type: "GET",
			url: "/levels",
			data: {},
			dataType: "json",
			success: function(data){
				//console.log("levels:", data)
				section.server_levels = data
				section.update_levels()
				h1e.trigger_redraw()
			},
			error: function(xhr, type){
			}
		});
	} else {
		section.update_levels()
		h1e.trigger_redraw()
	}

	this.draw = function(h1e){
		//h1e.draw_rect(0, 0, SCREEN_W, SCREEN_H, "rgba(0, 0, 0, 0.9)")
		//h1e.draw_text(90, 8, "Select Level", {fgcolor:"#ffffff"})
		//h1e.draw_text(85, 8, "Level Selection", {fgcolor:"#888888"})

		var special_selected = false
		var level = section.levels[section.selected_i]
		if(level){
			if(level.type === "generate")
				special_selected = true
		}

		for(var i=0; i<section.levels.length; i++){
			var level = section.levels[i]
			var yi = Math.floor(i / section.levels_per_row)
			var xi = i - yi * section.levels_per_row
			var x0 = xi * 30
			var y0 = yi * 30 + 15
			//console.log(""+i+":", level, " x0:",x0, " y0:",y0)
			if(special_selected){
				y0 += 10
			} else {
				if(yi >= Math.floor(section.selected_i / section.levels_per_row))
					y0 += 10
			}
			if(section.selected_i == i){
				h1e.draw_rect(x0+0, y0+0, 30-0, 30-0, "#888888",
						{disable_cdt_autohide:true})
				h1e.draw_rect(x0+1, y0+1, 30-2, 30-2, "#000000",
						{disable_cdt_autohide:true})
			}
			if(level.type === "generate"){
				var opts = {fontsize:16, fgcolor:"#888888"}
				for(var y1=0; y1<GEN_W; y1++)
				for(var x1=0; x1<GEN_H; x1++){
					var i1 = y1*GEN_W + x1
					h1e.draw_sprite(x0+2+x1*3, y0+2+y1*3, "small_bg")
				}
				h1e.draw_text(x0+9, y0+15, "?", opts)
			}
			if(level.type === "saved"){
				var data = level.data
				for(var y1=0; y1<data.h; y1++)
				for(var x1=0; x1<data.w; x1++){
					var i1 = y1*data.w + x1
					var t = data.tiles[i1]
					var tn = char_to_tiletype(t)
					h1e.draw_sprite(x0+2+x1*3, y0+2+y1*3, "small_"+tn)
				}
				if(!special_selected){
					if(yi == Math.floor(section.selected_i / section.levels_per_row)){
						var opts = {right_align:true, fontsize:6}
						if(section.selected_i != i)
							opts.fgcolor = "#333333"
						var s = "?"
						if(level.best_moves !== undefined)
							s = ""+level.best_moves
						h1e.draw_text(x0+28, y0-8, s, opts)
					}
				}
			}
			;(function(level){
				h1e.add_clickable_draw_target([x0, y0, 30, 30], function(){
					var game = create_game_from_level(level)
					game_section.reset_game(game)
					h1e.remove_section(section)
				})
			})(level)
		}
		var x0 = 0
		var y0 = 15
		var opts = {fgcolor:"#333333"}
		var level = section.levels[section.selected_i]
		if(level){
			if(level.type === "generate"){
				h1e.draw_text(x0, y0, "Generate new level", opts)
			}
			/*if(level.type === "saved"){
				h1e.draw_text(x0, y0, "Best moves: ?", opts)
				//if(level.best_moves && level.best_moves.length > 0)
			}*/
		}
		h1e.draw_text(69, 160-8, "Wandstep for LD31 by 8Dromeda",
				{fgcolor:"#333333", fontsize:6})
	}
	this.event = function(h1e, event){
		if(event.type == "keydown_repeatable"){
			if(h1e.iskey(event.key, ["escape", "q", "l"])){
				h1e.remove_section(this)
				return true
			}
			if(h1e.iskey(event.key, ["left", "a"])){
				if(section.selected_i > 0)
					section.selected_i -= 1
				else
					section.selected_i = section.levels.length-1
				LEVEL_SELECTED_I = section.selected_i
				return true
			}
			if(h1e.iskey(event.key, ["right", "d"])){
				if(section.selected_i < section.levels.length-1)
					section.selected_i += 1
				else
					section.selected_i = 0
				LEVEL_SELECTED_I = section.selected_i
				return true
			}
			if(h1e.iskey(event.key, ["up", "w"])){
				if(section.selected_i >= section.levels_per_row)
					section.selected_i -= section.levels_per_row
				LEVEL_SELECTED_I = section.selected_i
				return true
			}
			if(h1e.iskey(event.key, ["down", "s"])){
				if(section.selected_i < section.levels.length-section.levels_per_row)
					section.selected_i += section.levels_per_row
				LEVEL_SELECTED_I = section.selected_i
				return true
			}
			if(h1e.iskey(event.key, ["enter", "space", "pagedown"])){
				var game = create_game_from_level(
						section.levels[section.selected_i])
				game_section.reset_game(game)
				h1e.remove_section(this)
				return true
			}
		}
	}
	this.update = function(h1e){
		return false // Always redraw manually
	}
}

var game_section = new GameSection()
h1e.push_section(game_section)
if(window.location.protocol !== "file:"){
	h1e.push_section(new LevelSelectSection(game_section))
}

function pad_stuff(){
	h1e.resize_canvas($(window).width() - 5, $(window).height() - 5)
}

$(document).ready(function(){
	pad_stuff()
	$(window).resize(function(){
		pad_stuff()
	})
	/*$("#audio_b")[0].volume = 0.7
	$("#audio_b")[0].play()
	$("#audio_b")[0].addEventListener('ended', function(){
		this.currentTime = 0;
		this.play();
	}, false);*/
	try{
		if(window.localStorage){
			var s = window.localStorage.getItem("sound_disabled")
			if(s == "true")
				$("#audio_b")[0].pause()
		}
	}catch(e){
	}
	h1e.start()
})

